@@ -64,6 +64,14 @@ macro_rules! scope_impls {
                     _ => Scope::Other(s)
                 }
             }
+
+            /// Get the scope as a borrowed string.
+            pub fn as_str(&self) -> &str {
+                match self {
+                    $(Scope::$i => $rename,)*
+                    Self::Other(c) =>  c.as_ref()
+                }
+            }
         }
 
     };
@@ -83,6 +91,7 @@ scope_impls!(
     ChannelManageVideos,      scope: "channel:manage:videos",      doc: "Manage a channel’s videos, including deleting videos.";
     ChannelModerate,          scope: "channel:moderate",           doc: "Perform moderation actions in a channel. The user requesting the scope must be a moderator in the channel.";
     ChannelReadEditors,       scope: "channel:read:editors",       doc: "View a list of users with the editor role for a channel.";
+    ChannelReadGoals,         scope: "channel:read:goals",         doc: "View Creator Goals for a channel.";
     ChannelReadHypeTrain,     scope: "channel:read:hype_train",    doc: "View Hype Train information for a channel.";
     ChannelReadPolls,         scope: "channel:read:polls",         doc: "View a channel’s polls.";
     ChannelReadPredictions,   scope: "channel:read:predictions",   doc: "View a channel’s Channel Points Predictions.";
@@ -108,13 +117,8 @@ scope_impls!(
     WhispersRead,             scope: "whispers:read",              doc: "View your whisper messages.";
 );
 
-impl Scope {
-    /// Get [Scope] as an oauth2 Scope
-    pub fn as_oauth_scope(&self) -> oauth2::Scope { oauth2::Scope::new(self.to_string()) }
-}
-
-impl From<oauth2::Scope> for Scope {
-    fn from(scope: oauth2::Scope) -> Self { Scope::parse(scope.to_string()) }
+impl std::borrow::Borrow<str> for Scope {
+    fn borrow(&self) -> &str { self.as_str() }
 }
 
 impl From<String> for Scope {
